#!/usr/bin/perl
# Program name: nmon2web.pl
# Purpose - convert nmon.csv file(s) into web pages
# Author - Bruce Spencer
# Disclaimer:  this provided "as is".  
# Date - June 2006
#
# July 2006 Bug fix
# 	Typo's = wlmcpu.rdd -> wlmcpu.rrd, $$WEIGHT -> $WEIGHT
# 	Logic = rrd_update 
#	
# Sept 2006 Bug Fix
# 	Added rrd_template for nmon 11e 
# Nov 2, 2006 
#	Added *.csv files for nmon input (*.nmon and *.csv)
# 	Modified LPAR charts
#	Replace missing data with 0's
#	Linux on Intel:  use hostname for serial number
#	  Created sub mk_graph_linux for Linux
#	  Changed mem.rrd, lpar.rrd, 
# Dec 8, 2006
#	Fixed $NCPU
#	  Typo "of -> Of": $NCP=&get_setting("lsconf,\"Number Of Processors",...
#	Added filter for to change "not available" adapter to "NA"
# May 15, 2007
#	Added $UTC_START_DAY to allow hourly nmon's.  Needed to start at 00:00
# June 22, 2007
#	Fix wrong title on WLM Memory chart
#	Fix wlmcpu for dedicatated lpars

$nmon2web_ver="1.0.3c June 2007";

use Time::Local;

# This program provides the following functions
#  1. Converts nmon daily performance data (from multiple
#	partitions) into web pages on a centralized web server
#  2. Tracks long term partition performance
#  3. Aggregates performance data across a partitioned server
#  4. Provides configuration and AIX tuning change control
#
#  This was written for AIX servers, but will work with linux as well (except
#  the configuraton/change control)
#
#  Prereq's 
#  1. Web server
#     a. Web server software
#     b. perl
#     c. "rrdtool"  public domain "round robin" database and graphing tool
#  2. AIX or Linux servers
#     a. nmon V11 (nmmon 9, will work....sorta)
# 
#  Process
#  1. Use "nmon -f" on AIX/Linux partitions to collect performance data
#     a. nmon csv files for a partition must sort in time order
#  2. Upload nmon data to web server (your choice ftp, nfs, scp, ....)
#     a. Directory = $NMON_DIR   (this program recurses subdirectories)
#  3. Cron job on web server converts csv into rrdtool database and web pages
#  4. Point web browser to provided index.html file
#  
#  Key files/directories
#  A. nmon files
#	1. Must have ".nmon" filename extension 
#	2. Location on web server = $NMON_DIR (or subdir). 
# 	   a. User id that runs this program must have read access
#	3. After processing, the nmon files are gzip'd (save space, prevents 
#	   future processing. 
#  B. Web server directories
#	1. $HTTP_DIR is the parent directory for the HTML files
#	   a. this program must have read/write to this directory
#	   b. This directory must be accessable to the web server
#       2. $HTTP_DIR Files 
#
#	     $HTTP_DIR/index.html     - Entry point for viewing data
#	     $HTTP_DIR/server_list.js - list of servers in Javascript format.
#
#	3. Server web subdirectories 
#		$SERVER_DIR = $HTTP_DIR/$SN/$TYPE/$SERVER/
#		$SN = system serial number (multiple LPARS, micropartitions
#			 may be associated with one serial number)
#		$TYPE = partition type:  Dedicated|Shared
#		$SERVER = hostname of partition
#
# 	     	$SERVER_DIR/rrddata - rrd database files
# 	     	$SERVER_DIR/rrddata/update - update files for db's in rrddata
# 	     	$SERVER_DIR/$YEAR/$MONTH/$DAY - daily perf charts
# 	        $SERVER_DIR/trend - long term trend charts
#	        $SERVER_DIR/Shared/sharedpool - temp dir for aggregated charts
#		     a. cgi program must have read write permission
# 

#################################################
## 	Your Customizations Go Here            ##
#################################################

#  Source directory for nmon csv files
$NMON_DIR="/home/baspence/nmon";  	
#$HTTP_DIR="/var/www/html/nmon"; # Absolute path to index.html
$HTTP_DIR="/home/baspence/public_html/nmon"; # Absolute path to index.html

$DB_MONTHS=6;  # rrdtool: number of months to retain data before rrd wraps


# End "Your Customizations Go Here".  
# You're on your own, if you change anything beyond this line :-)

####################################################################
#############		Main Program 			############
####################################################################

# Initialize common variables
&initialize;

# Process all "nmon" files located in the $NMON_DIR
@nmon_files=`ls $NMON_DIR/*.nmon $NMON_DIR/*.csv 2>/dev/null`;

if (@nmon_files eq 0 ) { die ("No nmon or csv files found in $NMON_DIR\n"); }
chomp(@nmon_files);

foreach $FILENAME ( @nmon_files ) {

  $start=time();
  @now=localtime($start);
  $now=join(":",@now[2,1,0]);
  print ("$now: Begin processing file = $FILENAME\n");
  
  # Parse nmon file, skip if unsuccessful
  if (( &get_nmon_data ) gt 0 ) { next; }
  $now=time();
  $now=$now-$start;
  print ("\t$now: Finished get_nmon_data\n");
  
  # Initial setup for new servers (ie create directories, rrd databases
  if ( ! -d $SERVER_DIR ) { 
	&setup_new_server; 
  	$now=time();
  	$now=$now-$start;
	print ("\t$now: Finished setup_new_server\n");
	}
  
  # Make a directory for today's charts
  if ( ! -d $DAILY_DIR) { system("mkdir", "-p", "$DAILY_DIR");  }

  # Get first and last update times
  # Get rrd "start date". Use for long term chart.
  if ( -f $JOURNAL ) {
        open(FILE,"$JOURNAL");
  	@journal=<FILE>;
  	close(FILE);
  	$UTC_RRD_START=(split(/:/,$journal[0]))[1]; # first rrd update
  	$lastupdate=(split(/:/,$journal[@journal]))[1];
  } else { 
  	$UTC_RRD_START=$UTC_START;  # Today
  	$lastupdate=0;
  }

  # Do not continue if data out of time sequence
  if ( $lastupdate > $UTC_START ) { next; }

  # Process "normal" rrd data (standard data)
  # Store these in $SERVER_DIR/rrddata
  &update_rrddata;
  $now=time();
  $now=$now-$start;
  print ("\t$now: Finished update_rrddata\n");

  # Create non-normal rrd data (ie "top 10 processes, disks")
  # Store these in the $SERVER_DIR/$YR/$MO/$DAY
  &update_dynamic_rrddata;
  $now=time();
  $now=$now-$start;
  print ("\t$now: Finished update_dynamic_rrd\n");
  
  # Update (or create) server_list.js file for index.html
  # This is javascript data that tells index.html where the charts reside
  &server_list_js;   

  # Create graphs (gif files) Syntax: mk_rrdgraph(dir, start date [,end date])
  if ($AIXVER =~ /AIX/) {
    &mk_rrdgraph($TREND_DIR,$UTC_RRD_START);  
    &mk_rrdgraph($DAILY_DIR,$UTC_START_DAY);	 
  } else { # LINUX
    &mk_rrdgraph_linux($TREND_DIR,$UTC_RRD_START);  
    &mk_rrdgraph_linux($DAILY_DIR,$UTC_START_DAY);	 
  }

  $now=time();
  $now=$now-$start;
  print ("\t$now: Finished mk_rrd_graphs\n");
  
  # Change control for AIX configuration/tuning 
  # Save current settings: non default, look for changes
  &change_control;
  $now=time();
  $now=$now-$start;
  print ("\t$now: Finished change_control\n");

  # Create index.html file for new daily and "trend" directory 
  &mk_index($TREND_DIR,"$HOSTNAME Long Term Performance");
  &mk_index($DAILY_DIR,"$HOSTNAME Performance $DATE");
  $now=time();
  $now=$now-$start;
  print ("\t$now: Finished mk_index\n");
  
  # Compress the file to save space, and to remove it from future processing 
  system("gzip","$FILENAME");

  @now=localtime(time());
  $now=join(":",@now[2,1,0]);
  print ("$now: End processing $FILENAME\n-------------------------\n");
  
} # end foreach nmon_files
exit 0;
  
############################################
#############  Subroutines 	############
############################################

##################################################################
## mk_rrdgraph: Create rrdtool command file (rrd_graph)		##
##################################################################
sub mk_rrdgraph {

# This subroutine creates the rrd commands to create graphs
# from 3 types of input variables 
#   1.  Static   - always the same variables (ie %CPU, %mem)
#   2.  Variable - depends on config (ie number/type adapters, disks)
#   3.  Special  - specific variables, or data from multiple rrd databases
#   
#   There is a 4th type of daily chart (unique) not processed here. 
#   It includes the top 10 processes/disks for the day. 
#   This type of chart is created in the daily updates subroutine

my ($OUTDIR,$START)=@_;
my $dir;
my @cols;
my $i;
my @wlm;
my $key;
my $rrdfile;
my $x;
my $y;
my $type;
my $category;
my $graph;

chdir("$OUTDIR");
unless (open(RRDGRAPH, ">rrd_graph")) { die("Can not open $OUTDIR/rrd_graph\n"); };


# Create rrdtool commands to create graphs fom static variables

print RRDGRAPH (qq|graph cpu_all.gif -r -l 0 -u 100 -v "Percent Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Over all CPU Utilization" DEF:User=$RRD_DIR/cpu_all.rrd:UserPct:AVERAGE AREA:User#0000FF:"User%" DEF:Sys=$RRD_DIR/cpu_all.rrd:SysPct:AVERAGE STACK:Sys#00FF00:"Sys%" DEF:Wait=$RRD_DIR/cpu_all.rrd:WaitPct:AVERAGE STACK:Wait#FF0000:"Wait%" DEF:Idle=$RRD_DIR/cpu_all.rrd:IdlePct:AVERAGE STACK:Idle#FFFF00:"Idle%" \n|);

print RRDGRAPH (qq|graph system_file.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "File System functions" DEF:iget=$RRD_DIR/file.rrd:iget:AVERAGE LINE3:iget#FF0000:"iget" DEF:namei=$RRD_DIR/file.rrd:namei:AVERAGE LINE3:namei#00FF00:"namei" DEF:dirblk=$RRD_DIR/file.rrd:dirblk:AVERAGE LINE3:dirblk#0000FF:"dirblk" \n|);

print RRDGRAPH (qq|graph system_filerw.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "File System read/write per Second" DEF:readch=$RRD_DIR/file.rrd:readch:AVERAGE LINE3:readch#FF0000:"readch" DEF:writech=$RRD_DIR/file.rrd:writech:AVERAGE LINE3:writech#00FF00:"writech" \n|);

print RRDGRAPH (qq|graph system_forkexec.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls(fork/exec) per Second" DEF:fork=$RRD_DIR/proc.rrd:fork:AVERAGE LINE3:fork#FF0000:"fork" DEF:exec=$RRD_DIR/proc.rrd:exec:AVERAGE LINE3:exec#00FF00:"exec" \n|);

print RRDGRAPH (qq|graph system_ipc.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls(sem/msg) per Second" DEF:sem=$RRD_DIR/proc.rrd:sem:AVERAGE LINE3:sem#FF0000:"sem" DEF:msg=$RRD_DIR/proc.rrd:msg:AVERAGE LINE3:msg#00FF00:"msg" \n|);

if ( -f "$RRD_DIR/lpar.rrd" && $TYPE=~/Shared/ ) {

print RRDGRAPH (qq|graph cpu_abs.gif -r -l 0 -v "Absolute CPU" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Absolute CPU Utilization vs Entitlement" DEF:CPU=$RRD_DIR/lpar.rrd:PhysicalCPU:AVERAGE LINE3:CPU#0000FF:"Total CPU" DEF:ENTITLE=$RRD_DIR/lpar.rrd:entitled:AVERAGE LINE3:ENTITLE#FF0000:"Entitled" \n|);

print RRDGRAPH (qq|graph cpu_lpar2.gif -r -l 0 -v "Physical CPUs, Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Shared Pool Utilization" DEF:CPU=$RRD_DIR/lpar.rrd:PhysicalCPU:AVERAGE AREA:CPU#0000FF:"Phys_CPU_Used_by_Part" DEF:OtherLPARs=$RRD_DIR/lpar.rrd:OtherLPARs:AVERAGE STACK:OtherLPARs#999999:"OtherLPARs" DEF:PoolIdle=$RRD_DIR/lpar.rrd:PoolIdle:AVERAGE STACK:PoolIdle#FFFF00:"Idle_CPUs_in_Pool" DEF:Entitled=$RRD_DIR/lpar.rrd:entitled:AVERAGE LINE3:Entitled#000000:"Partition_Entitlement" DEF:Pool=$RRD_DIR/lpar.rrd:poolCPUs:AVERAGE LINE3:Pool#FF0000:"CPU_Pool_Size" \n|); 

}

print RRDGRAPH (qq|graph mem1.gif -r -l 0 -u 100 -v "Percent" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Memory Free Percent" DEF:RealFree=$RRD_DIR/mem.rrd:Real_Free_Pct:AVERAGE LINE3:RealFree#0000FF:"RealFree" \n|);

print RRDGRAPH (qq|graph mem2.gif -r -l 0 -u 100 -v "Percent" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Virtual Memory Free Percent" DEF:VirtualFree=$RRD_DIR/mem.rrd:Virtual_free_Pct:AVERAGE LINE3:VirtualFree#0000FF:"VirtualFree" \n|);

print RRDGRAPH (qq|graph mem3.gif -r -l 0 -v "MBytes" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Memory Free Size" DEF:RealFreeMB=$RRD_DIR/mem.rrd:Real_free_MB:AVERAGE LINE3:RealFreeMB#0000FF:"RealFreeMB" \n|);

print RRDGRAPH (qq|graph mem4.gif -r -l 0 -v "MBytes" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Virtual Memory Free Size" DEF:VirtualFreeMB=$RRD_DIR/mem.rrd:Virtual_free_MB:AVERAGE LINE3:VirtualFreeMB#0000FF:"VirtualFreeMB" \n|);

print RRDGRAPH (qq|graph mem5.gif -r -l 0 -v "MBytes" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Whole Memory Size" DEF:RealMB=$RRD_DIR/mem.rrd:Real_total_MB:AVERAGE LINE3:RealMB#0000FF:"Real MB" \n|);

print RRDGRAPH (qq|graph mem6.gif -r -l 0 -v "MBytes" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Whole Virtual Memory Size" DEF:VirtualMB=$RRD_DIR/mem.rrd:Virtual_total_MB:AVERAGE LINE3:VirtualMB#0000FF:"VirtualMB" \n|);

# print RRDGRAPH (qq|graph mem9.gif -r -l 0 -u 100 -v "Percent" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Memory Percent Filesystem Cache" DEF:maxperm=$RRD_DIR/memuse.rrd:maxpermPct:AVERAGE LINE3:maxperm#0000FF:"maxperm%" DEF:minperm=$RRD_DIR/memuse.rrd:minpermPct:AVERAGE LINE3:minperm#00FF00:"minperm%" DEF:numperm=$RRD_DIR/memuse.rrd:numpermPct:AVERAGE LINE3:numperm#0000FF:"numperm%" \n|);
# print RRDGRAPH (qq|graph mem10.gif -r -l 0 -v "Blocks" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Memory Freelist" DEF:minfree=$RRD_DIR/memuse.rrd:minfree:AVERAGE LINE3:minfree#0000FF:"minfree" DEF:maxfree=$RRD_DIR/memuse.rrd:maxfree:AVERAGE LINE3:maxfree#00FF00:"maxfree" \n|);

print RRDGRAPH (qq|graph memnew.gif -r -l 0 -u 100 -v "Percent Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Memory Usage Percent (MEMNEW)" DEF:Process=$RRD_DIR/memnew.rrd:ProcessPct:AVERAGE AREA:Process#FF0000:"Process%" DEF:FScache=$RRD_DIR/memnew.rrd:FScachePct:AVERAGE STACK:FScache#00FF00:"FScache%" DEF:System=$RRD_DIR/memnew.rrd:SystemPct:AVERAGE STACK:System#0000FF:"System%" \n|);

print RRDGRAPH (qq|graph page1.gif -r -l 0 -v "Operations per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Paging VMM Stats" DEF:faults=$RRD_DIR/page.rrd:faults:AVERAGE LINE3:faults#0000FF:"faults" DEF:reclaims=$RRD_DIR/page.rrd:reclaims:AVERAGE LINE3:reclaims#00FF00:"reclaims" DEF:scans=$RRD_DIR/page.rrd:scans:AVERAGE LINE3:scans#0000FF:"scans" DEF:cycles=$RRD_DIR/page.rrd:cycles:AVERAGE LINE3:cycles#FFFF00:"cycles" \n|);

print RRDGRAPH (qq|graph page2.gif -r -l 0 -v "Pages per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Pagespace Paging" DEF:pgsin=$RRD_DIR/page.rrd:pgsin:AVERAGE LINE3:pgsin#0000FF:"pgsin" DEF:pgsout=$RRD_DIR/page.rrd:pgsout:AVERAGE LINE3:pgsout#00FF00:"pgsout" \n|);

print RRDGRAPH (qq|graph page3.gif -r -l 0 -v "Pages per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Filesystem Paging" DEF:pgin=$RRD_DIR/page.rrd:pgin:AVERAGE LINE3:pgin#0000FF:"pgin" DEF:pgout=$RRD_DIR/page.rrd:pgout:AVERAGE LINE3:pgout#00FF00:"pgout" \n|);

print RRDGRAPH (qq|graph system_runq.gif -r -l 0 -v "Process on Queue" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Run Queue" DEF:Runnable=$RRD_DIR/proc.rrd:Runnable:AVERAGE LINE3:Runnable#0000FF:"Runnable" \n|);

print RRDGRAPH (qq|graph system_switch.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Process Switch" DEF:pswitch=$RRD_DIR/proc.rrd:pswitch:AVERAGE LINE3:pswitch#0000FF:"pswitch" \n|);

print RRDGRAPH (qq|graph system_readwrite.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls(read/write)" DEF:read=$RRD_DIR/proc.rrd:read:AVERAGE LINE3:read#0000FF:"read" DEF:write=$RRD_DIR/proc.rrd:write:AVERAGE LINE3:write#00FF00:"write" \n|);

print RRDGRAPH (qq|graph page_swapin.gif -r -l 0 -v "Operations per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Swapin" DEF:Swapin=$RRD_DIR/proc.rrd:Swap-in:AVERAGE LINE3:Swapin#0000FF:"Swapin" \n|);

print RRDGRAPH (qq|graph system_syscall.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls" DEF:syscall=$RRD_DIR/proc.rrd:syscall:AVERAGE LINE3:syscall#0000FF:"syscall" \n|);

print RRDGRAPH (qq|graph system_aio.gif -r -l 0 -v "Async I/O" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Asynchronous I/O" DEF:aioprocs=$RRD_DIR/procaio.rrd:aioprocs:AVERAGE LINE3:aioprocs#0000FF:"aioprocs" DEF:aiorunning=$RRD_DIR/procaio.rrd:aiorunning:AVERAGE LINE3:aiorunning#00FF00:"aiorunning" DEF:aiocpu=$RRD_DIR/procaio.rrd:aiocpu:AVERAGE LINE3:aiocpu#0000FF:"aiocpu" \n|);


# Create rrdtool commands to create graphs from "variable" variables
# (ie configuraton differences like number of disks, adapters)
# This section plots all variables 
# It identifies all variables by quering the rrdtool database

@variable_graphs =(
qq|$DAILY_DIR/diskbusy.rrd:graph diskbusy.gif -r -l 0 -u 100 -v "% Busy" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Disk %Busy" |,
qq|$DAILY_DIR/diskbsize.rrd:graph diskbsize.gif -r -l 0 -v "Block Size" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Block Size" |,
qq|$DAILY_DIR/diskread.rrd:graph diskread.gif -r -l 0 -v "KBytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Read (KB/s)" |,
qq|$DAILY_DIR/diskread.rrd:graph diskreadtotal.gif -r -l 0 -v "KBytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Read (KB/s Stacked)" |,
qq|$DAILY_DIR/diskwrite.rrd:graph diskwrite.gif -r -l 0 -v "KBytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Writes KBps" |,
qq|$DAILY_DIR/diskwrite.rrd:graph diskwritetotal.gif -r -l 0 -v "KBytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Write (KB/s Stacked)" |,
qq|$DAILY_DIR/diskxfer.rrd:graph diskxfer.gif -r -l 0 -v "Transfers per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Transfers per Second" |,
qq|$DAILY_DIR/diskxfer.rrd:graph diskxfertotal.gif -r -l 0 -v "Transfers per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Transfers per Second Stacked" |,
qq|$RRD_DIR/neterror.rrd:graph neterror.gif -r -l 0 -v "Errors/Collisions per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network Errors" |,
qq|$RRD_DIR/net.rrd:graph net.gif -r -l 0 -v "Kbytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network" |,
qq|$RRD_DIR/netpacket.rrd:graph netpacket.gif -r -l 0 -v "packets per sec" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network Read/Write Packets per Second" |,
qq|$RRD_DIR/net.rrd:graph nettotal.gif -r -l 0 -v "Kbytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network (KB/s Stacked)" |,
qq|$DAILY_DIR/top.rrd:graph top.gif -r -l 0 -u 100  -v "% CPU" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "%CPU by Top Processes" |,
);

# Get the variables for each chart from the rrdtool db file
# print the rrdtool command
foreach $line ( @variable_graphs) {

  ($rrdfile,$graph)=split(/:/,$line);
  $x=get_rrd_ds("$rrdfile");    # query the rrdtool db for variables
  @cols=split(/,/,$x);   	# Variable names

  # Area charts: first def type  = AREA, all else = STACK
  print RRDGRAPH ("$graph ");
  if ($graph =~ /Stack/) { $type="AREA";} else { $type="LINE3";}
  for ($i=0; $i < @cols; $i++){
      print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE $type:$cols[$i]$color[$i]:"$cols[$i]" |);
      if ($graph =~ /AREA/) { $type="STACK";} # Change for remaining AREA DS's
  } # end for
  print RRDGRAPH ("\n");

 } # end foreach

# Special graphs
# selected variables, or data from multiple databases

print RRDGRAPH (qq|graph ioadapter1.gif -r -l 0 -v "Kbytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Disk Adapter KBps" |,);
$rrdfile="$RRD_DIR/ioadapt.rrd";
$x=get_rrd_ds("$rrdfile");   # list of variables in database
@cols=split(/,/,$x);
$type="LINE3";
for ($i=0; $i < @cols; $i++){
  if ($cols[$i] =~ /xfer/) { next; } # data rates only
  print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE $type:$cols[$i]$color[$i]:"$cols[$i]" |);
  } # end for
print RRDGRAPH ("\n");

print RRDGRAPH (qq|graph iototal.gif -r -l 0 -v "Kbytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Disk Adapter per Second (Stacked)" |,);
$rrdfile="$RRD_DIR/ioadapt.rrd";
$x=get_rrd_ds("$rrdfile");   # list of variables in database
@cols=split(/,/,$x);
$type="AREA";
for ($i=0; $i < @cols; $i++){
  if ($cols[$i] =~ /xfer/) { next; } # data rates only
  print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE $type:$cols[$i]$color[$i]:"$cols[$i]" |);
  $type="STACK";
} # end for
print RRDGRAPH ("\n");

if ( -f "$RRD_DIR/wlmcpu.rrd" ) { 

if ( $TYPE =~ /Shared/ ) {

  print RRDGRAPH ("graph wlmcpu_abs.gif -r -l 0 -v \"CPUs\" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title \"Absolute CPU Usage by WLM Class\" DEF:cpu=$RRD_DIR/lpar.rrd:PhysicalCPU:AVERAGE " );
  $rrdfile="$RRD_DIR/wlmcpu.rrd";
  $x=get_rrd_ds("$rrdfile");   # list of variables in database
  @cols=split(/,/,$x);
  for($i=1;$i<@cols;$i++){
    print RRDGRAPH (" DEF:P$i=$rrdfile:$cols[$i]:AVERAGE ");
  }
  for($i=1;$i<@cols;$i++){
    print RRDGRAPH (" CDEF:$cols[$i]=cpu,P$i,\*,0.01,\* ");
  }
  for($i=1;$i<@cols;$i++){
    print RRDGRAPH (qq| LINE3:$cols[$i]$color[$i]:"$cols[$i]" |);
  }

} else { # Dedicated CPU partitions

  print RRDGRAPH ("graph wlmcpu_abs.gif -r -l 0 -u 100 -v \"%CPU\" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title \"%CPU Usage by WLM Class\" " );
  $rrdfile="$RRD_DIR/wlmcpu.rrd";
  $x=get_rrd_ds("$rrdfile");   # list of variables in database
  @cols=split(/,/,$x);
  for($i=1;$i<@cols;$i++){
    print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE |);
    print RRDGRAPH (qq| LINE3:$cols[$i]$color[$i]:"$cols[$i]" |);
  }

} # end if

print RRDGRAPH ("\n");

print RRDGRAPH (qq|graph wlmbio.gif -r -l 0 -v \"IOs\" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title \"Absolute Block IO by WLM Class\" | );
$rrdfile="$RRD_DIR/wlmbio.rrd";
$x=get_rrd_ds("$rrdfile");   # list of variables in database
@cols=split(/,/,$x);
for($i=1;$i<@cols;$i++){
  print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE |);
  print RRDGRAPH (qq| LINE3:$cols[$i]$color[$i]:"$cols[$i]" |);
}
print RRDGRAPH ("\n");

print RRDGRAPH (qq|graph wlmmem.gif -r -l 0 -u 100 -v \"Percent\" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title \"% Memory by WLM Class\" | );
$rrdfile="$RRD_DIR/wlmmem.rrd";
$x=get_rrd_ds("$rrdfile");   # list of variables in database
@cols=split(/,/,$x);
for($i=1;$i<@cols;$i++){
  print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE |);
  print RRDGRAPH (qq| LINE3:$cols[$i]$color[$i]:"$cols[$i]" |);
}
print RRDGRAPH ("\n");
}

close(RRDGRAPH);

# Execute rrdtool commands the create the gif files
`rrdtool - < rrd_graph`;

} # end mk_rrdgraph


##################################################################
## mk_rrdgraph_linux: Create rrdtool command file for linux    ### 
##################################################################
sub mk_rrdgraph_linux {

# This subroutine creates the rrd commands to create graphs
# from 3 types of input variables 
#   1.  Static   - always the same variables (ie %CPU, %mem)
#   2.  Variable - depends on config (ie number/type adapters, disks)
#   3.  Special  - specific variables, or data from multiple rrd databases
#   
#   There is a 4th type of daily chart (unique) not processed here. 
#   It includes the top 10 processes/disks for the day. 
#   This type of chart is created in the daily updates subroutine

my ($OUTDIR,$START)=@_;
my $dir;
my @cols;
my $i;
my @wlm;
my $key;
my $rrdfile;
my $x;
my $y;
my $type;
my $category;
my $graph;

chdir("$OUTDIR");
unless (open(RRDGRAPH, ">rrd_graph")) { die("Can not open $OUTDIR/rrd_graph\n"); };


# Create rrdtool commands to create graphs fom static variables

print RRDGRAPH (qq|graph cpu_all.gif -r -l 0 -u 100 -v "Percent Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Over all CPU Utilization" DEF:User=$RRD_DIR/cpu_all.rrd:UserPct:AVERAGE AREA:User#0000FF:"User%" DEF:Sys=$RRD_DIR/cpu_all.rrd:SysPct:AVERAGE STACK:Sys#00FF00:"Sys%" DEF:Wait=$RRD_DIR/cpu_all.rrd:WaitPct:AVERAGE STACK:Wait#FF0000:"Wait%" DEF:Idle=$RRD_DIR/cpu_all.rrd:IdlePct:AVERAGE STACK:Idle#FFFF00:"Idle%" \n|);

print RRDGRAPH (qq|graph system_forkexec.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls(fork/exec) per Second" DEF:fork=$RRD_DIR/proc.rrd:fork:AVERAGE LINE3:fork#FF0000:"fork" DEF:exec=$RRD_DIR/proc.rrd:exec:AVERAGE LINE3:exec#00FF00:"exec" \n|);

print RRDGRAPH (qq|graph system_ipc.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls(sem/msg) per Second" DEF:sem=$RRD_DIR/proc.rrd:sem:AVERAGE LINE3:sem#FF0000:"sem" DEF:msg=$RRD_DIR/proc.rrd:msg:AVERAGE LINE3:msg#00FF00:"msg" \n|);

if ( -f "$RRD_DIR/lpar.rrd" && $TYPE=~/Shared/ ) {

print RRDGRAPH (qq|graph cpu_abs.gif -r -l 0 -v "Absolute CPU" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Absolute CPU Utilization vs Entitlement" DEF:CPU=$RRD_DIR/lpar.rrd:PhysicalCPU:AVERAGE LINE3:CPU#0000FF:"Total CPU" DEF:ENTITLE=$RRD_DIR/lpar.rrd:partition_entitled_capacity:AVERAGE LINE3:ENTITLE#FF0000:"Entitled" \n|);

print RRDGRAPH (qq|graph cpu_lpar2.gif -r -l 0 -v "Physical CPUs, Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Shared Pool Utilization" DEF:CPU=$RRD_DIR/lpar.rrd:PhysicalCPU:AVERAGE AREA:CPU#0000FF:"Phys_CPU_Used_by_Part" DEF:OtherLPARs=$RRD_DIR/lpar.rrd:OtherLPARs:AVERAGE STACK:OtherLPARs#999999:"OtherLPARs" DEF:PoolIdle=$RRD_DIR/lpar.rrd:PoolIdle:AVERAGE STACK:PoolIdle#FFFF00:"Idle_CPUs_in_Pool" DEF:Entitled=$RRD_DIR/lpar.rrd:partition_entitled_capacity:AVERAGE LINE3:Entitled#000000:"Partition_Entitlement" DEF:Pool=$RRD_DIR/lpar.rrd:pool_capacity:AVERAGE LINE3:Pool#FF0000:"CPU_Pool_Size" \n|); 

}

print RRDGRAPH (qq|graph mem1.gif -r -l 0 -v "Memory MB" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Free Memory" DEF:RealFree=$RRD_DIR/mem.rrd:memfree:AVERAGE LINE3:RealFree#0000FF:"memfree" DEF:Total=$RRD_DIR/mem.rrd:memtotal:AVERAGE LINE3:Total#FFFFFF:"memtotal" \n|);

print RRDGRAPH (qq|graph page1.gif -r -l 0 -v "Memory MB" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Free Memory" DEF:SwapTotal=$RRD_DIR/mem.rrd:swaptotal:AVERAGE LINE3:SwapTotal#FF0000:"swaptotal" DEF:SwapCached=$RRD_DIR/mem.rrd:swapcached:AVERAGE LINE3:SwapCached#0000FF:"swapcached" DEF:SwapFree=$RRD_DIR/mem.rrd:swapfree:AVERAGE LINE3:SwapFree#00FF00:"swapfree" \n|);

print RRDGRAPH (qq|graph system_runq.gif -r -l 0 -v "Process on Queue" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Run Queue" DEF:Runnable=$RRD_DIR/proc.rrd:Runnable:AVERAGE LINE3:Runnable#0000FF:"Runnable" \n|);

print RRDGRAPH (qq|graph system_switch.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Process Switch" DEF:pswitch=$RRD_DIR/proc.rrd:pswitch:AVERAGE LINE3:pswitch#0000FF:"pswitch" \n|);

print RRDGRAPH (qq|graph system_readwrite.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls(read/write)" DEF:read=$RRD_DIR/proc.rrd:read:AVERAGE LINE3:read#0000FF:"read" DEF:write=$RRD_DIR/proc.rrd:write:AVERAGE LINE3:write#00FF00:"write" \n|);

print RRDGRAPH (qq|graph page_swapin.gif -r -l 0 -v "Operations per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Swapin" DEF:Swapin=$RRD_DIR/proc.rrd:Swap-in:AVERAGE LINE3:Swapin#0000FF:"Swapin" \n|);

print RRDGRAPH (qq|graph system_syscall.gif -r -l 0 -v "Per Second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "System Calls" DEF:syscall=$RRD_DIR/proc.rrd:syscall:AVERAGE LINE3:syscall#0000FF:"syscall" \n|);


# Create rrdtool commands to create graphs from "variable" variables
# (ie configuraton differences like number of disks, adapters)
# This section plots all variables 
# It identifies all variables by quering the rrdtool database

@variable_graphs =(
qq|$DAILY_DIR/diskbusy.rrd:graph diskbusy.gif -r -l 0 -u 100 -v "% Busy" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Disk %Busy" |,
qq|$DAILY_DIR/diskbsize.rrd:graph diskbsize.gif -r -l 0 -v "Block Size" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Block Size" |,
qq|$DAILY_DIR/diskread.rrd:graph diskread.gif -r -l 0 -v "KBytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Read (KB/s)" |,
qq|$DAILY_DIR/diskread.rrd:graph diskreadtotal.gif -r -l 0 -v "KBytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Read (KB/s Stacked)" |,
qq|$DAILY_DIR/diskwrite.rrd:graph diskwrite.gif -r -l 0 -v "KBytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Writes KBps" |,
qq|$DAILY_DIR/diskwrite.rrd:graph diskwritetotal.gif -r -l 0 -v "KBytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Write (KB/s Stacked)" |,
qq|$DAILY_DIR/diskxfer.rrd:graph diskxfer.gif -r -l 0 -v "Transfers per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Transfers per Second" |,
qq|$DAILY_DIR/diskxfer.rrd:graph diskxfertotal.gif -r -l 0 -v "Transfers per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "DISK Transfers per Second Stacked" |,
qq|$RRD_DIR/net.rrd:graph net.gif -r -l 0 -v "Kbytes per second" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network" |,
qq|$RRD_DIR/netpacket.rrd:graph netpacket.gif -r -l 0 -v "packets per sec" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network Read/Write Packets per Second" |,
qq|$RRD_DIR/net.rrd:graph nettotal.gif -r -l 0 -v "Kbytes per second Stacked" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "Network (KB/s Stacked)" |,
qq|$DAILY_DIR/top.rrd:graph top.gif -r -l 0 -u 100  -v "% CPU" --start $START --end $UTC_END --width $WIDTH --height $HEIGHT --title "%CPU by Top Processes" |
);

# Get the variables for each chart from the rrdtool db file
# print the rrdtool command
foreach $line ( @variable_graphs) {

  ($rrdfile,$graph)=split(/:/,$line);
  $x=get_rrd_ds("$rrdfile");    # query the rrdtool db for variables
  @cols=split(/,/,$x);   	# Variable names

  # Area charts: first def type  = AREA, all else = STACK
  print RRDGRAPH ("$graph ");
  if ($graph =~ /Stack/) { $type="AREA";} else { $type="LINE3";}
  for ($i=0; $i < @cols; $i++){
      print RRDGRAPH (qq| DEF:$cols[$i]=$rrdfile:$cols[$i]:AVERAGE $type:$cols[$i]$color[$i]:"$cols[$i]" |);
      if ($graph =~ /AREA/) { $type="STACK";} # Change for remaining AREA DS's
  } # end for
  print RRDGRAPH ("\n");

 } # end foreach

print RRDGRAPH ("\n");

close(RRDGRAPH);

# Execute rrdtool commands the create the gif files
`rrdtool - < rrd_graph`;

} # end mk_rrdgraph_linux


####################################
## Create/update server_list.js   ##
## Required for index.html	  ##
####################################
sub server_list_js {

my @js;  # Javascript file for index.html
my $i;

# Month has to be in 2 digit format, 01, 02, ....
if ( length($MON) eq 1 ) { $MON="0" . $MON;} 

if ( -f "$HTTP_DIR/server_list.js" ){ 	

    ### File server_list.js exists ###
    open(OUTFILE, "$HTTP_DIR/server_list.js");
    @js=<OUTFILE>;
    close(OUTFILE);

    # Add or update timestamp for server update
    for($i=0;$i<@js;$i++){  
	
	if ($js[$i] =~ /\|$HOSTNAME\|/ ) {  # Update

	   # Host found, change last timestamp field (col4)
	   @cols=split(/\|/,$js[$i]);
	   $cols[4]="$YR$MON$DAY\"\n";
	   $js[$i]=join('|',@cols); 
           unless (open(OUTFILE, ">$HTTP_DIR/server_list.js")) { 
		die ("Unable to open $HTTP_DIR/server_list.js "); }
	   print OUTFILE (@js);
	   close (OUTFILE);
	   return(0); 

	} elsif ( $js[$i] =~ /\)\;/ ) { # Add host

	   $js[$i]=",\"$SN|$TYPE|$HOSTNAME|$YR$MON$DAY|$YR$MON$DAY\"\n";
	   $js[$i+1]=');';   
           unless (open(OUTFILE, ">$HTTP_DIR/server_list.js")) { 
		die ("Unable to open $HTTP_DIR/server_list.js "); }
	   print OUTFILE (@js);
	   close (OUTFILE);
	   return(0);

	} # end if

    }# end for

} else {
    ## Create js file (one time only)
    unless (open(OUTFILE, ">$HTTP_DIR/server_list.js")) { die ("Unable to open $HTTP_DIR/server_list.js "); }

print OUTFILE <<END_JS;
// List of servers for index.htm file
// Format:  server|first archive date (yyyymmdd)
var servers= new Array(
"$SN|$TYPE|$HOSTNAME|$YR$MON$DAY|$YR$MON$DAY"
);
END_JS
close(OUTFILE);

} # end if

} # end mk_server_list

#############################################################################
## mk_rrd_databases: Create initial rrd databases for a new server	   ##
#############################################################################
sub mk_rrd_databases {

# Input format: nmon csv data has the following format:
#	category,title,var1[,var2,var3,....]	

# There are 4 input types: This subroutine processes 3 of the 4 types:
#
#  1.  Static - var1,var2,... are constant (ie %cpu, %mem)
#  2.  Static/nmon - nmon version differences
#  3.  Static/config - config differences (ie number of adapters)
#
#  The following databases are created in rrd_update, not here
#  4.  Uniq - changes from day to day (ie top processes/disks)
#
#  Type 1-3-  are typically static over time (after initial setup)
# 	      These rrd databases for these are located in $RRD_DIR
#  Type 2 - use a different temmplate for each nmon version
#  Type 3 - this program queries rrdtool database for var names
#  Type 4 - not handled by this subroutine. 
#		These rrd databases or stored in $DAILY_DIR
#
#  Other considerations:
#  1. nmon variable names may not be compatible with rrdtool var names
#     rrdtool restrictions = [0-9a-zA-Z_]  (must remove [- +()]

my $i;
my @cols;
my $category;
my $rrdfile;
my $variable;
my $heading;
my $basemaxrows;
my $dailysteps;
my $dailymaxrows;
my $weeklysteps;
my $weeklymaxrows;

#############################################################################
##  Commands for creating rrdtool databases in directory $RRD_DIR
##  The rrd_template contains the category, title, optional variable names
##     $rrd_template[category]="title,var1,var2,....."
##
##  The following syntax implies we must fill in the vars by
##  quering the rrdtool database
##     $rrd_template[category]="title,"
##
##  You can list the nmon variable names using the following command
##    cat nmon.csv | cut -f1 -d, |sort -u | while read x
##	>do
##      >grep $x nmon.csv | head -1
##      >done
##
##  You can get the variable names out of a rrdtool database by
##     rrdtool info file_name | grep ds | cut -f1 -d, | sort -u
##
#############################################################################

# Input Type2 = variations in nmon versions
if ( $AIXVER eq "-1" ) {

%rrd_template= (
'CPU_ALL','CPU Total,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
"DISKBSIZE",'Disk Block Size,',
"DISKBUSY",'Disk PctBusy,',
"DISKREAD",'Disk Read kb/s,',
"DISKWRITE",'Disk Write kb/s,',
"DISKXFER",'Disk transfers per second,',
"LPAR","Shared CPU LPAR Stats,PhysicalCPU,capped,shared_processor_mode,system_potential_processors,system_active_processors,pool_capacity,MinEntCap,partition_entitled_capacity,partition_max_entitled_capacity,MinProcs,partition_active_processors,partition_potential_processors,capacity_weight,unallocated_capacity_weight,BoundThrds,MinMem,unallocated_capacity,pool_idle_time,OtherLPARs",
"MEM","Memory MB,memtotal,hightotal,lowtotal,swaptotal,memfree,highfree,lowfree,swapfree,memshared,cached,active,bigfree,buffers,swapcached,inactive",
"NET","Network I/O,",
"NETPACKET","Network Packets,",
"PROC","Processes,Runnable,Swapin,pswitch,syscall,read,write,fork,exec,sem,msg",
"TOP","+PID,Time,PctCPU,PctUsr,PctSys,Size,ResSet,ResText,ResData,ShdLibMajorFault,MinorFault,Command"
 ) ;

} elsif ( $NMONVER =~ /9/ ) {

%rrd_template=(

'CPU_ALL','CPU Total,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
"IOADAPT","Disk Adapter,",
'MEM','Memory,Real_Free_Pct,Virtual_free_Pct,Real_free_MB,Virtual_free_MB,Real_total_MB,Virtual_total_MB',
'MEMUSE','Memory Use Percent,numpermPct,minpermPct,maxpermPct,minfree,maxfree',
"NET","Network I/O,",
"NETPACKET","Network Packets,",
"PAGE","Paging prdibm1b,faults,pgin,pgout,pgsin,pgsout,reclaims,scans,cycles",
"PROC","Processes prdibm1b,Runnable,Swap-in,pswitch,syscall,read,write,fork,exec,rcvint,xmtint,sem,msg",
"PROCAIO","Asynchronous I/O prdibm1b,aioprocs,aiorunning,aiocpu",
'WLMBIO','Block IO percent ,',
'WLMCPU','CPU percent ,',
'WLMMEM','Memory percent ,'
);

} elsif ( $NMONVER =~ /10/  ||  $NMONVER =~ /11b/ ) {

%rrd_template=(

'CPU_ALL','CPU Total,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
'CPU_ENTITLE','CPU Total Percent ,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
'CPU_PHYSICAL','CPU Total Percent,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
'CPU_EC_USE','CPU Entitled Percent,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
'ESSREAD','ESS Disk Read,',
'ESSWRITE','ESS Disk Write,',
'ESSXFER','ESS Disk XFER,',
'FILE','File I/O,iget,namei,dirblk,readch,writech,ttyrawch,ttycanch,ttyoutch',
'IOADAPT','Disk Adapter,',
'LPAR','Logical Partition ,PhysicalCPU,virtualCPUs,logicalCPUs,poolCPUs,entitled,weight,PoolIdle,usedAllCPUPct,usedPoolCPUPct,SharedCPU,OtherLPARs',
'MEM','Memory,Real_Free_Pct,Virtual_free_Pct,Real_free_MB,Virtual_free_MB,Real_total_MB,Virtual_total_MB',
'MEMNEW','Memory New,ProcessPct,FScachePct,SystemPct,FreePct,PinnedPct,UserPct',
'MEMUSE','Memory Use Percent,numpermPct,minpermPct,maxpermPct,minfree,maxfree,numclient,maxclient',
'NET','Network I/O (KBps),',
'NETERROR','Network Errors,',
'NETPACKET','Network Packets (Reads/writes per second),',
'PAGE','Paging ,faults,pgin,pgout,pgsin,pgsout,reclaims,scans,cycles',
'PAGING','PagingSpace MB Free,',
'PROC','Processes ,Runnable,Swap-in,pswitch,syscall,read,write,fork,exec,sem,msg',
'PROCAIO','Asynchronous I/O,aioprocs,aiorunning,aiocpu',
'WLMBIO','Block IO percent ,',
'WLMCPU','CPU percent ,',
'WLMMEM','Memory percent ,'

);

} else {  # default: nmon 11d+

%rrd_template=(

'CPU_ALL','CPU Total,UserPct,SysPct,WaitPct,IdlePct,Busy,CPUs',
'FILE','File I/O,iget,namei,dirblk,readch,writech,ttyrawch,ttycanch,ttyoutch',
'IOADAPT','Disk Adapter,',
'LPAR','Logical Partition,PhysicalCPU,virtualCPUs,logicalCPUs,poolCPUs,entitled,weight,PoolIdle,usedAllCPUPct,usedPoolCPUPct,SharedCPU,Capped,EC_UserPct,EC_SysPct,EC_WaitPct,EC_IdlePct,VP_UserPct,VP_SysPct,VP_WaitPct,VP_IdlePct,OtherLPARs',
'MEM','Memory,Real_Free_Pct,Virtual_free_Pct,Real_free_MB,Virtual_free_MB,Real_total_MB,Virtual_total_MB',
'MEMNEW','Memory New,ProcessPct,FScachePct,SystemPct,FreePct,PinnedPct,UserPct',
'MEMUSE','Memory Use Percent,numpermPct,minpermPct,maxpermPct,minfree,maxfree,numclient,maxclient',
'NET','Network I/O (KBps),',
'NETERROR','Network Errors,',
'NETPACKET','Network Packets (Reads/writes per second),',
'PAGE','Paging ,faults,pgin,pgout,pgsin,pgsout,reclaims,scans,cycles',
'PAGING','PagingSpace MB Free,',
'PROC','Processes ,Runnable,Swap-in,pswitch,syscall,read,write,fork,exec,sem,msg',
'PROCAIO','Asynchronous I/O,aioprocs,aiorunning,aiocpu',
'WLMBIO','Block IO percent ,',
'WLMCPU','CPU percent ,',
'WLMMEM','Memory percent ,'

);
} # end if


## Input Type 2 - get var's by querying the rrdtool data file

 foreach $category ( keys ( %rrd_template)) {

  # Trailing /,',$/ signifies "dynamic". Need to get variable list from nmon 
  if ($rrd_template{$category} !~ /,$/ ) { next; }  
  $heading=(grep(/^$category\,/, @nmon))[0];
  if ( length($heading) eq 0 ) { next; }  # skip if heading not found

  $heading=~s/\%/Pct/g;
  $heading=~s/not available/NA/g;
  $heading=~s/[^_0-9a-zA-Z,]*//g;  #edit nmon name to make rrd compatible
  @cols=split(/,/,$heading);
  $rrd_template{$category}= $rrd_template{$category} . join(',',@cols[2..(@cols -1)]);

 } # end foreach 
 
unless (open(RRDCREATE, ">$RRD_DIR/rrd_create")) { die("Can not open $RRD_DIR/rrd_create\n"); }

# Convert database retention time $DB_MONTHS, to rrd format
$basemaxrows=int($DB_MONTHS*30*24*3600/$INTERVAL);
$dailysteps=(24*3600/$INTERVAL);   # steps/day
$dailymaxrows=int($DB_MONTHS*12);
$weeklysteps=int(7*24*3600/$INTERVAL);     # steps/week
$weeklymaxrows=int($DB_MONTHS*4.3);	 

foreach $category (sort (keys ( %rrd_template ))) {

  $rrdfile=lc($category). "\.rrd";
  @cols=split(/,/, $rrd_template{$category});

  # Create rrd_create statement
  print RRDCREATE ("create $rrdfile --start $UTC_START --step $INTERVAL");
  for ($i=1; $i<@cols; $i++) { 
	print RRDCREATE (" DS:$cols[$i]:GAUGE:$MAXINTERVAL:U:U"); 
  }

  print RRDCREATE (" RRA:AVERAGE:0.5:1:$basemaxrows");
  print RRDCREATE (" RRA:AVERAGE:0.5:$dailysteps:$dailymaxrows");
  print RRDCREATE (" RRA:AVERAGE:0.5:$weeklysteps:$weeklymaxrows\n");

} # end foreach
close(RRDCREATE);
chdir("$RRD_DIR");
`rrdtool - < rrd_create`;

} # end mk_rrd_database

############################################################
## Read perf data from nmon file, add to rrd data base    ##
############################################################
sub update_rrddata {

my @updates;
my @cols;
my $update;
my $category;
my $i;
my $ncols;
my $rrdfile;
my @rrdfiles;
my $TS;
 
unless (open(RRDUPDATE, ">$RRD_UPDATE_FILE")) { die("Can not open $RRD_UPDATE_FILE\n"); }

chdir("$RRD_DIR");
@rrdfiles=`ls *rrd`;
chomp(@rrdfiles);

foreach $rrdfile (sort @rrdfiles) {

  $category=uc((split(/\./,$rrdfile))[0]);
  @updates=grep(/^$category,/, @nmon);

  # skip TOC and first data line
  for ($i=2; $i<@updates; $i++) {

    $TS=$UTC_START + $INTERVAL*($i);
    $update="update $rrdfile $TS:";

    @cols=split(/,/, $updates[$i]);
    # Convert blank "Busy" to 0
    for ($j=0; $j<@cols; $j++){
        if ($cols[$j] == "") { $cols[$j] = 0; }
    }

    # Calculate the OtherLPARs field for Shared Pool graph
    #if ($category eq "LPAR") { $cols[@cols-1] = $cols[5]-$cols[2]-$cols[8]; }
    if ($category eq "LPAR") { $cols[@cols] = $cols[5]-$cols[2]-$cols[8]; }

    $update .= join(':',@cols[2..(@cols-1)]); 

    print RRDUPDATE ("$update\n");

  } # end for i
} # end for $category

close(RRDUPDATE);
chdir("$SERVER_DIR/rrddata");
`rrdtool - < $RRD_UPDATE_FILE`;

unless (open(RRDUPDATE, ">>$JOURNAL")) { die("Can not open $JOURNAL\n"); }
print RRDUPDATE ("$DATE:$UTC_START:$UTC_END\n");
close(RRDUPDATE);

} # end updated_rrddata

########################################################
###	Get an nmon setting from csv file            ###
###	finds first occurance of $search             ###
###	Return the selected column...$return_col     ###
###	Syntax:                                      ###
###     get_setting($search,$col_to_return,$separator)##
########################################################

sub get_setting {

my $i;
my $value="-1";
my ($search,$col,$separator)= @_;    # search text, $col, $separator

for ($i=0; $i<@nmon; $i++){

  if ($nmon[$i] =~ /$search/ ) {
	$value=(split(/$separator/,$nmon[$i]))[$col];
	$value =~ s/["']*//g;  #remove non alphanum characters
	return($value);
	} # end if

  } # end for

return($value);

} # end get_setting

#####################
##  Clean up       ##
#####################
sub clean_up_line {

	# remove characters not compatible with nmon variable
	# Max rrdtool variable length is 19 chars
	# Variable can not contain special characters (% - () )
	my ($x)=@_;	

	# print ("clean_up, before: $i\t$nmon[$i]\n");
	$x =~ s/\%/Pct/g;
	# $x =~ s/\W*//g;
	$x =~ s/\/s/ps/g;   	# /s  - ps
	$x =~ s/\//s/g;		# / - s
	$x =~ s/\(/_/g;
	$x =~ s/\)/_/g;
	$x =~ s/ /_/g;
	$x =~ s/-/_/g;
	$x =~ s/_KBps//g;
	$x =~ s/_tps//g;
	$x =~ s/[:,]*\s*$//;
	$retval=$x; 

} # end clean up
	

##########################################
##  Extract headings from nmon csv file ##
##########################################
sub initialize {

%MONTH2NUMBER =  ("jan", 1, "feb",2, "mar",3, "apr",4, "may",5, "jun",6, "jul",7, "aug",8, "sep",9, "oct",10, "nov",11, "dec",12 );

@MONTH2ALPHA =  (	"junk","jan", "feb", "mar", "apr", "may", "jun", "jul", "aug", "sep", "oct", "nov", "dec" );

# Chart Variables
$HEIGHT=300; # pixels
$WIDTH=800;

# Random colors for disk charts
@color=("#0000FF","#00FF00","#FF0000","#000000","#FFFF00","#FF00FF","#00FFFF","#800000","#008000","#000080","#808000","#800080","#008080","#FF8000","#FF0080","#80FF00","#00FF80","#8000FF","#0080FF","#80FFFF","#FF80FF","#FFFF80","#505000","#500050","#005050","#F0F0F0","#0F0F0F","#0F0000","#0F0F00","#0F000F");  # line colors


} # end initialize

# Get data from nmon file, extract specific data fields (hostname, date, ...)
sub get_nmon_data {

my $key;
my $x;
my $category;
my %toc;
my @cols;

# Read nmon file
unless (open(FILE, $FILENAME)) { return(1); }
@nmon=<FILE>;  # input entire file
close(FILE);
chomp(@nmon);

# Cleanup nmon data remove trailing commas and colons
for($i=0; $i<@nmon;$i++ ) {
    $nmon[$i] =~ s/[:,]*\s*$//;
}

# Get nmon/server settings (search string, return column, delimiter)
$AIXVER		=&get_setting("AIX",2,",");
$DATE		=&get_setting("date",2,",");
$ENT_CPU	=&get_setting(qq|^BBBL.*Entitled Capacity|,3,",");
$NCPU		=&get_setting(qq|lsconf,"Number Of Processors|,3,",");
$NCPU		=(split(/:\s+/, $NCPU))[1];
$HOSTNAME	=&get_setting("host",2,",");

$INTERVAL	=&get_setting("interval",2,","); # nmon sampling interval
$MAXINTERVAL=2*$INTERVAL;  # rrdtool sample time. assumes missing data if >

$MEMORY		=&get_setting(qq|lsconf,"Good Memory Size:|,1,":");
$MODEL		=&get_setting("modelname",3,'\s+');
$NMONVER	=&get_setting("version",2,",");
if ( $NMONVER =~ /v9/ ) { print("Warning: this program is intended for nmon v10+.  This file is nmon v9. Consider upgrading.\n"); }

$SNAPSHOTS	=&get_setting("snapshots",2,",");  # number of readings

$STARTTIME	=&get_setting("AAA,time",2,",");
($HR, $MIN)=split(/\:/,$STARTTIME);


if ($AIXVER eq "-1") {
	$SN=$HOSTNAME; 	# Probably a Linux host
} else {
	$SN	=&get_setting("systemid",4,",");
	$SN 	=(split(/\s+/,$SN))[0]; # "systemid IBM,SN ..."
}

$TYPE		=&get_setting("^BBBP.*Type",3,",");
if ( $TYPE =~ /Shared/ ) { $TYPE="Shared"; } else { $TYPE="Dedicated"; }

$MODE		=&get_setting("^BBBP.*Mode",3,",");
$MODE		=(split(/: /, $MODE))[1];
# $MODE		=~s/\"//g;

$ENTITLE	=&get_setting("^BBBP.*Entitled Capacity",3,",");
$ENTITLE	=(split(/: /, $ENTITLE))[1];
# $ENTITLE	=~s/\"//g;

$WEIGHT		=&get_setting("^BBBP.*Variable Capacity Weight",3,",");
$WEIGHT		=(split(/: /, $WEIGHT))[1];
# $WEIGHT		=~s/\"//g;

$POOL		=&get_setting("^BBBP.*Active CPUs in Pool",3,",");
$POOL		=(split(/: /, $POOL))[1];
# $POOL		=~s/\"//g;

# Calculate UTC time (seconds since 1970)
# NMON V9  dd/mm/yy
# NMON V10+ dd-MMM-yyyy

if ( $DATE =~ /[a-zA-Z]/ ) {   # Alpha = assume dd-MMM-yyyy date format
	($DAY, $MMM, $YR)=split(/\-/,$DATE);
	$MMM=lc($MMM);
	$MON=$MONTH2NUMBER{$MMM};
} else {
	($DAY, $MON, $YR)=split(/\//,$DATE);
	$YR=$YR + 2000;
	$MMM=$MONTH2ALPHA[$MON];
} # end if


## Setup Working directories/files
# server subdirectory 
$SERVER_DIR="$HTTP_DIR/$SN/$TYPE/$HOSTNAME";

# Create file and subdirectory names 
$CONFIG_DIR		= "$SERVER_DIR/config";
$RRD_DIR      		= "$SERVER_DIR/rrddata";
$JOURNAL		= "$RRD_DIR/update_journal.txt";
$RRD_UPDATE_FILE 	= "$RRD_DIR/updates/rrd_update\.$YR$MMM$DAY";
$TREND_DIR		= "$SERVER_DIR/trend"; 
$DAILY_DIR		= "$SERVER_DIR/$YR/$MMM/$DAY"; 
$SHAREDPOOL		= "$SERVER_DIR/sharedpool";
$UNIQUE_RRD		= "$DAILY_DIR/rrd_create_dynamic";

## Calculate UTC time (seconds since 1970).  Required format for the rrdtool.

##  timelocal format
##    day=1-31
##    month=0-11
##    year = x -1900  (time since 1900) (seems to work with either 2006 or 106)

$m=$MON - 1;  # jan=0, feb=2, ...

# print ("Date=$DATE, MMM=$MMM, Month=$MON, Month-1= $m, Day=$DAY, Year=$YR, Hour=$HR, Min=$MIN\n"); 

$UTC_START=timelocal(0,$MIN,$HR,$DAY,$m,$YR); 
$UTC_START_DAY=timelocal(0,0,0,$DAY,$m,$YR); 
$UTC_END=$UTC_START + $INTERVAL * $SNAPSHOTS;
return(0);

} # end get_nmon_data

###############################################
## Create the index.html for the server perf
###############################################
sub mk_index {

my ($OUTDIR, $TITLE)=@_;
my $i;
my @cols;
my @x;
my $lastline;
my @config;
my $href;
my $title;
my $grp;
my @gifs;

chdir($OUTDIR);
unless (open(OUT, ">index.html")) { die("Can not open $OUTDIR/index.html\n"); }

print OUT<<EOF;
<HTML><TITLE>$TITLE</TITLE><BODY>
<center>
<table width="100%">
<tr bgcolor="blue">
<td align="center">
<font color="white">
<h1><a name="top">$TITLE</a></h1>
</font>
</td></tr>
<tr>
<td>

<table>
<tr>
<td>Hostname
<td>$HOSTNAME
</tr><tr>
<td>DATE
<td>$DATE
</tr><tr>
<td>Model
<td>$MODEL
</tr><tr>
<td>Serial Number
<td>$SN
</tr><tr>
<td>AIX Version
<td>$AIXVER
</tr><tr>
<td>NMON Version
<td>$NMONVER
</tr><tr>
<td>nmon2web.pl Ver
<td>$nmon2web_ver
</tr><tr>
<td>Partition Type
<td>$TYPE	
</tr><tr>
<td>Micropartition CPU
<td>$MODE
</tr><tr>
<td>Entitled CPU
<td>$ENTITLE
</tr><tr>
<td>Uncapped Weight
<td>$WEIGHT
</tr><tr>
<td>CPU Pool Size
<td>$POOL
</tr><tr>
<td>Number CPU's 
<td>$NCPU
</tr><tr>
<td>Memory (MB)
<td>$MEMORY
</tr></table>

</td></tr></table>
</center>
<p>
<table border=1 cellspacing=0 color=brown>
<tr><td valign="top" >
<font scale="-2">
EOF

## Chart Categories
# Use key from %chart to categorize filenames
my %chart= (
	"cpu","CPU ",
	"io","Adapter ",
	"lpar","LPAR ",
	"mem","Memory ",
	"net","Network ",
	"page","Paging ",
	"system","System",
	"top","Processes",
	"wlm","WLM"
);
if ($OUTDIR eq $DAILY_DIR) { $chart{"disk"}="Disk";}

# Links of left column
foreach $key (sort ( keys %chart)) {
  print OUT (qq|\[<a href="\#${key}">$chart{$key}</a>\]\n|);
}
print OUT (qq|\[<a href="\#config">Config</a>\]\n|);
print OUT (qq|</font></td><td>|);

@gifs=`ls *.gif`;
chomp(@gifs);

foreach $key (sort ( keys %chart)) {

  print OUT (qq|<h2><a  name="${key}">$chart{$key} Statistics</a></h2>\n|);
  foreach  $i (@gifs) {
   if ( $i =~ /^$key/ ){ print OUT (qq|<img src=$i alt=$i>\n|); }
  }
  print OUT (qq|<br><a href="\#top">Top</a>\n|);
}
print OUT (qq|</td></tr></table><hr>\n|);

@config=`cat $CONFIG_DIR/current.cfg`;
print OUT ("@config");

print OUT ("</pre></body></html>\n");
close(OUT);

} # end mk_index
######################
## Setup New Server ##
######################
sub setup_new_server {

	system("mkdir","-p", "$TREND_DIR");  	# for long term charts
	system("mkdir","-p", "$CONFIG_DIR");
	system("touch","$CONFIG_DIR/change.log");
	system("mkdir","-p", "$RRD_DIR/updates"); # for rrd update files

	
	if ( $TYPE =~ /Shared/ && ! -d "$HTTP_DIR/$SN/Shared/sharedpool" ) {
	   system("mkdir","-p", "$HTTP_DIR/$SN/Shared/sharedpool");
	   chmod(0777,"$HTTP_DIR/$SN/Shared/sharedpool"); #writeable by cgi
	   open(FILE,">$HTTP_DIR/$SN/Shared/sharedpool/virtual.cfg.templ");
	   print FILE <<EOF;
# Customizing Summary Charts for Combined Virtual SCSI and Etherent 
#
# The virtual.cfg file allows you to override the default behavior of 
# the $CGI-BIN/nmon2web.cgi program, which creates the combined 
# performance charts.  By default, the cgi program looks for vscsi devices on
# any partition (LPAR or micropartition) and en0 interfaces on micropartitions.
#
# To override, create a "virtual.cfg" file in this directory.
# Include 1 line per partition (remove the comment #)
# server_name,vdev1[,vdev2,...]
# Examples
larry,vscis0,vscsi1,en3
moe,vscsi0,en1,en2
curly,en1
EOF
close(FILE);
	}

	&mk_rrd_databases;	     # Create rrd databases

}# end new server

# Change control
sub change_control {

  my @config;
  my @group;
  my $i;
  my $j;
  my $x;
  my $BACK='<p><a href=#top>Top</a>';
  my $heading="NAME                      CUR    DEF    BOOT   MIN    MAX    UNIT           TYPE";
  my @cols;

  # @tuning = "href:title:search"
  # list of settings to display
  my @tuning=(	"#tmem:Virtual Memory Settings:BBBP.*,vmo",
		"#tio:I/O Settings:BBBP.*,ioo",
		"#tsched:Schedo Settings:BBBP.*,schedo",
		"#tnet:Network Settings:BBBP.*,no",
		"#tnfs:NFS Settings:BBBP.*,nfso"
  );

  chdir($CONFIG_DIR);
  unless ( open(OUT,">current.cfg") ) { 
	print "change_control: Can not open file current.cfg";
	return(1);
	}


  print OUT (qq|<h2><a name="config">Configuration</a></h2><pre>\n|);

  @config=grep(/^AAA/, @nmon);
  for($i=0;$i<@config;$i++){
	if ( $config[$i] =~ /date/ ) { next; }
	if ( $config[$i] =~ /time/ ) { next; }
	$config[$i] =~ s/\"//g;
	$config[$i] =~ s/AAA,//g;
	print OUT ("$config[$i]\n");
  }

  @config=grep(/lsconf/,@nmon);
  for($i=0;$i<@config;$i++){
	@cols=split(/\,/,$config[$i]);
	$cols[3] =~ s/\"//g;
	print OUT ("$cols[3]\n"); # config data
  }

  print OUT (qq|</pre><h2><a name="#tuning">Tuning</a> (Non-default settings)</h2>\n|);
  print OUT (qq|<p><a href="#config">[Config</a>][<a href="#tmem">Memory</a>][<a href="#tio">IO</a>][<a href="#tsched">Sched</a>][<a href="#tnet">Network</a>][<a href="#tnfs">NFS</a>]\n|);

  for ($j=0;$j<@tuning;$j++){

    ($href,$title,$grp)=split(/:/,$tuning[$j]);

    print OUT (qq|<h3><a name="$href">$title</a></h3>\n<pre>\n|);
    print OUT ("$heading\n");
    @config=grep(/$grp/,@nmon);
    
    for ($i=0;$i<@config;$i++){

	@cols=split(/\,/,$config[$i]);
	if ( $cols[3] =~ /^\"n---/ || $cols[3] =~ /^\"[A-Z\ ]/ ) { next; }
	$cols[3] =~ s/\"//g;

        # List non-default variables (var's start w/alpha, and have nCols>6)
        @x=split(/\s+/,$cols[3]);
        if ( @x > 5 &&  $x[1] ne $x[2] ) { 
	   print OUT ("$cols[3]\n"); }

    } # end for 

  print OUT ("</pre><hr>\n");  
} # end for $j
close(OUT);

open(OUT,"current.cfg");
@config=<OUT>;
close(OUT);

# Create last.cfg if doesn't exist
if ( ! -f "last.cfg") { 
	system("cp","current.cfg","last.cfg"); 
}
@group=`diff last.cfg current.cfg`;

print FILE ("$heading\n");
if ( @group gt 0 ) {
	open(FILE,">change.tmp");
	print FILE ("<h3> $DATE </h3>\n");
	for($i=0;$i<@group;$i++){

		if ( $group[$i] =~ /\</ ){
		   $group[$i] =~ s/\</Before: /;
			print FILE ("$group[$i]\n");
		} 
		elsif ($group[$i] =~ /\>/) {
		   $group[$i] =~ s/\>/After:  /;
			print FILE ("$group[$i]\n");
		} 
		# end if

	} 
	#end for

	close(FILE);
	`cat change.log >> change.tmp`;
	`mv change.tmp change.log`;

} 
#end if

open(FILE, ">index.html");

print FILE<<EOF;
<html><head><title>Configuration, Non Default Settings, Change Log</title></head><body>
<table width=100%>
<tr bgcolor=blue align=center>
<td>
<h1><a name=top><font color=white>$HOSTNAME Configuration/Tuning/Change Control</font></h1>
</tr></table>

<center>
<table><tr>
<td>Server
<td>$HOSTNAME
</tr><tr>
<td>Last Update
<td>$DATE
</tr><tr>
<td>Serial Number
<td>$SN
</tr><tr>
<td>Partition Type
<td>$TYPE
</tr></table>
</center>

<h2>Links</h2>
<p><a href=#current>Current Configuration</a>
<br><a href=#nondefault>AIX Settings</a>
<br><a href=#changelog>Change Log</a>
EOF

# Print config
print FILE ("@config");

print FILE ("</pre>$BACK\n");
print FILE ( qq|<h2><a name=changelog>Change Log</a></h2><pre>\n| );
@group=`cat change.log`;
print FILE ("@group");
print FILE ("</pre>$BACK\n");
print FILE ("</body></html>");
close(FILE);  


} 
# end change_control

## Create list of rrd variables in hash array ds ##
sub get_rrd_ds{
  my $db;
  my @var;
  my $line;
  
  $db=(@_)[0];
  @var=`rrdtool info $db |grep ds|cut -d. -f1|sort -u`;
  chomp(@var);
  for ($i=0;$i<@var;$i++){ 
    $var[$i]=~s/ds\[//; 
    $var[$i]=~s/\]//; 		
  }
  $line = join(",", @var);
  return($line);

} # end get_rrd_ds

# Create dynamic databases in $DAILY_DIR, and add data
sub update_dynamic_rrddata {

my $i;
my $j;
my @data;
my %var;
my $x;
my @cols;
my $last;
my $max=15;      # maximum number of processes to display
my $maxcol;
my $lastrecord;
my $lasttime;
my $toplist;
my $maxdisks=25;  # maximum number of disks to display
my @DISKS=("DISKBUSY","DISKBSIZE", "DISKREAD", "DISKWRITE", "DISKXFER");
my @top=();  #added 3/30/07

chdir($DAILY_DIR);

### Processes ###
### Create rrd database for today's top $max processes  ###
for($i=0,$j=0; $i<@nmon; $i++ ) {

 if ($nmon[$i] =~ /^TOP,\d+,T/) { 
     @cols=split(/,/,$nmon[$i]);
     $cols[13] =~ s/\W+//g; # remove non alphanum characters from var names
     $data[$j++]="$cols[2],$cols[3],$cols[13]"; # Time,%cpu,process name
     $var{$cols[13]}+=$cols[3];  # sum %CPU for determing top processes
   } # end if

} # end for

@data=sort(@data);

# Select the top $max processes by %CPU
# extract %cpu/var from associative array into regular array
$j=0;
foreach $key ( keys ( %var )) {
  $top[$j++]="$var{$key},$key";  # %cpu,varname 
} # end foreach

# numerical descending sort by %CPU (top processes at top of array)
@top=sort { $b <=> $a } @top;

# Take the top $max processes or #variables, whichever is smaller
if (@top < $max) {$j=@top;} else {$j=$max;}
@top=@top[0..$j-1];

# Keep the top process names, delete the %CPU column
for ($i=0;$i<@top;$i++) {

 	# print ("$i:  $top[$i]\n");
	$x=((split/,/,$top[$i]))[1];
	$top[$i]=$x; 
}


# rrdtool create command if file doesn't exist
if ( ! -f $DYNAMIC_RRD ) {
  open(RRDCREATE,">$UNIQUE_RRD");
  print RRDCREATE ("create top.rrd --start $UTC_START --step $INTERVAL");
  for ($i=0; $i<@top; $i++) { 
    print RRDCREATE (" DS:$top[$i]:GAUGE:$MAXINTERVAL:U:U");
  }
  print RRDCREATE (" RRA:AVERAGE:0.5:1:$SNAPSHOTS\n");
}

# Create rrd database updates for top processes...add to same file
# raw input data has 1 line per process ("T00n,$cpu,$var")

$toplist=join(" ",@top); # Store only variables in this list
for ($j=0;$j<@top;$j++) { $var{$top[$j]}=0; } 
$lasttime="T0001";
$lastrecord=@data - 1;
for ($i=0;$i<@data;$i++) {

  @cols=split(/,/,$data[$i]);
  if ( $cols[0] ne $lasttime || $i eq $lastrecord ){ 
     $x=$cols[0];
     $x=~s/T0*//; # strip the T000n to get the sequence number
     $TS=$UTC_START + $INTERVAL*($x);
     print RRDCREATE ("update top.rrd $TS");
     $x=0;
     for ($j=0;$j<@top;$j++) { 
	print RRDCREATE (":$var{$top[$j]}");
	$x+=$var{$top[$j]}; 
        $var{$top[$j]}=0; 
     } # end for
     print RRDCREATE ("\n");
     $lasttime=$cols[0];

  } elsif ( $toplist =~ /$cols[2]/ ) { # Keep if cols[2] is a top process
     $var{$cols[2]}=$cols[1];
  } # end if

} # end for


###################################
## Top Disk Performance
## Select the first $maxdisks
###################################

foreach $category ( @DISKS ) { 
  @data=grep(/^$category,/, @nmon);
  $category=lc($category);

  # Create database definition
  # @cols = category,title,var1,var2,...
  @cols=split(/,/,$data[0]);

  if ( @cols > $maxdisks + 2 ) { $maxcol=$maxdisks+2; } else { $maxcol=@cols; }

  # remove nonalphanum chars that are not compatible with rrdtool
  for ($i=2;$i<$maxcol ;$i++) { $cols[$i] =~ s/\W+//g; } 

  ##  Print rrdcreate commands for disk rrd databases
  print RRDCREATE ("create ${category}.rrd --start $UTC_START --step $INTERVAL");
  # $i=2 skip category, title columns
  for ($i=2; $i<$maxcol; $i++) { 
    print RRDCREATE (" DS:$cols[$i]:GAUGE:$MAXINTERVAL:U:U");
    }
  print RRDCREATE (" RRA:AVERAGE:0.5:1:$SNAPSHOTS\n");

  ## Print rrdupdate commands to add perf dat
  $maxcol--;  # adjust for join statement
  for ($i=2; $i<@data; $i++){
     $TS=$UTC_START + $INTERVAL*$i;
     print RRDCREATE ("update ${category}.rrd $TS");
     @cols=split(/,/,$data[$i]);
     $x=join(":",(@cols[2..$maxcol]));
     print RRDCREATE (":$x\n");
  } # end for

} # end foreach
close(RRDCREATE);
`rrdtool - <$UNIQUE_RRD`;


# print RRDCREATE (qq|graph top.gif -r -l 0 -u 100 -v "%CPU" --start $UTC_START --end $UTC_END --width 800 --height 300 --title "Top $max Processes (%CPU)" |);
# for ($i=1; $i < @top; $i++){
#     print RRDCREATE (qq| DEF:$top[$i]=top.rrd:$top[$i]:AVERAGE LINE3:$top[$i]$color[$i]:"$top[$i]" |);
#   } # end for
# print RRDCREATE ("\n");


} # end sub dynamic rrd
